@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright (c) 2009 - 2015 Detlev Offenbach <detlev@die-offenbachs.de>
+# Copyright (c) 2009 - 2016 Detlev Offenbach <detlev@die-offenbachs.de>
 #
 
 """
@@ -12,6 +12,7 @@ import bdb
 import os
 import atexit
 import inspect
+import ctypes
 from inspect import CO_GENERATOR
 
 from DebugProtocol import ResponseClearWatch, ResponseClearBreak, \
@@ -67,7 +68,6 @@ class DebugBase(bdb.Bdb):
         
         # current frame we are at
         self.currentFrame = None
-        self.currentFrameLocals = None
         
         # frame that we are stepping in, can be different than currentFrame
         self.stepFrame = None
@@ -100,14 +100,36 @@ class DebugBase(bdb.Bdb):
             the current frame (int)
         @return locals dictionary of the frame
         """
-        if frmnr:
-            f = self.currentFrame
-            while f is not None and frmnr > 0:
-                f = f.f_back
-                frmnr -= 1
-            return f.f_locals
-        else:
-            return self.currentFrameLocals
+        f = self.currentFrame
+        while f is not None and frmnr > 0:
+            f = f.f_back
+            frmnr -= 1
+        return f.f_locals
+    
+    def storeFrameLocals(self, frmnr=0):
+        """
+        Public method to store the locals into the frame, so an access to
+        frame.f_locals returns the last data.
+        
+        @keyparam frmnr distance of frame to store locals dictionary to. 0 is
+            the current frame (int)
+        """
+        cf = self.currentFrame
+        while cf is not None and frmnr > 0:
+            cf = cf.f_back
+            frmnr -= 1
+        
+        try:
+            if "__pypy__" in sys.builtin_module_names:
+                import __pypy__
+                __pypy__.locals_to_fast(cf)
+                return
+        except Exception:
+            pass
+        
+        ctypes.pythonapi.PyFrame_LocalsToFast(
+            ctypes.py_object(cf),
+            ctypes.c_int(0))
     
     def step(self, traceMode):
         """
@@ -583,7 +605,7 @@ class DebugBase(bdb.Bdb):
         stack = []
         while fr is not None:
             fname = self._dbgClient.absPath(self.fix_frame_filename(fr))
-            if not fname.startswith("<"):
+            if not fname.startswith("<") and not fname.endswith("bdb.py"):
                 fline = fr.f_lineno
                 ffunc = fr.f_code.co_name
                 
@@ -629,11 +651,13 @@ class DebugBase(bdb.Bdb):
         if self._dbgClient.mainFrame is None:
             if fn != self._dbgClient.getRunning():
                 return
-            self._dbgClient.mainFrame = frame
+            fr = frame
+            while (fr is not None and
+                    fr.f_code != self._dbgClient.handleLine.__code__):
+                self._dbgClient.mainFrame = fr
+                fr = fr.f_back
 
         self.currentFrame = frame
-        self.currentFrameLocals = frame.f_locals
-        # remember the locals because it is reinitialized when accessed
         
         fr = frame
         stack = []
@@ -645,7 +669,7 @@ class DebugBase(bdb.Bdb):
             # return from dispatch_call
             fr.f_trace = self.trace_dispatch
             fname = self._dbgClient.absPath(self.fix_frame_filename(fr))
-            if not fname.startswith("<"):
+            if not fname.startswith("<") and not fname.endswith("bdb.py"):
                 fline = fr.f_lineno
                 ffunc = fr.f_code.co_name
                 
@@ -709,14 +733,17 @@ class DebugBase(bdb.Bdb):
             try:
                 message = str(excval)
                 filename = excval.filename
-                linenr = excval.lineno
-                charnr = excval.offset
+                lineno = excval.lineno
+                charno = excval.offset
+                if charno is None:
+                    charno = 0
+                
+                exclist = [message, [filename, lineno, charno]]
+                filename = os.path.abspath(filename)
+                realSyntaxError = os.path.exists(filename)
             except (AttributeError, ValueError):
                 exclist = []
                 realSyntaxError = True
-            else:
-                exclist = [message, [filename, linenr, charnr]]
-                realSyntaxError = os.path.exists(filename)
             
             if realSyntaxError:
                 self._dbgClient.write("{0}{1}\n".format(
@@ -743,8 +770,6 @@ class DebugBase(bdb.Bdb):
             frlist.reverse()
             
             self.currentFrame = frlist[0]
-            self.currentFrameLocals = frlist[0].f_locals
-            # remember the locals because it is reinitialized when accessed
             
             for fr in frlist:
                 filename = self._dbgClient.absPath(self.fix_frame_filename(fr))
