@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright (c) 2009 - 2015 Detlev Offenbach <detlev@die-offenbachs.de>
+# Copyright (c) 2009 - 2016 Detlev Offenbach <detlev@die-offenbachs.de>
 #
 
 """
@@ -17,6 +17,7 @@ import time
 import imp
 import re
 import atexit
+import signal
 
 
 import DebugProtocol
@@ -25,6 +26,7 @@ from DebugBase import setRecursionLimit, printerr   # __IGNORE_WARNING__
 from AsyncFile import AsyncFile, AsyncPendingWrite
 from DebugConfig import ConfigVarTypeStrings
 from FlexCompleter import Completer
+from DebugUtilities import getargvalues, formatargvalues
 
 
 DebugClientInstance = None
@@ -372,6 +374,8 @@ class DebugClientBase(object):
                 filename = excval.filename
                 linenr = excval.lineno
                 charnr = excval.offset
+                if charno is None:
+                    charno = 0
             except (AttributeError, ValueError):
                 exclist = []
             else:
@@ -494,7 +498,7 @@ class DebugClientBase(object):
                 wd, fn, args, tracePython = arg.split('|')
                 self.__setCoding(fn)
                 sys.argv.append(fn)
-                sys.argv.extend(eval(args.replace("u'", "'")))
+                sys.argv.extend(eval(args))
                 sys.path = self.__getSysPath(os.path.dirname(sys.argv[0]))
                 if wd == '':
                     os.chdir(sys.path[1])
@@ -512,6 +516,7 @@ class DebugClientBase(object):
                 # set the system exception handling function to ensure, that
                 # we report on all unhandled exceptions
                 sys.excepthook = self.__unhandled_exception
+                self.__interceptSignals()
                 
                 # clear all old breakpoints, they'll get set after we have
                 # started
@@ -539,7 +544,7 @@ class DebugClientBase(object):
                 wd, fn, args = arg.split('|')
                 self.__setCoding(fn)
                 sys.argv.append(fn)
-                sys.argv.extend(eval(args.replace("u'", "'")))
+                sys.argv.extend(eval(args))
                 sys.path = self.__getSysPath(os.path.dirname(sys.argv[0]))
                 if wd == '':
                     os.chdir(sys.path[1])
@@ -557,6 +562,7 @@ class DebugClientBase(object):
                 # set the system exception handling function to ensure, that
                 # we report on all unhandled exceptions
                 sys.excepthook = self.__unhandled_exception
+                self.__interceptSignals()
                 
                 self.mainThread.tracePython = False
                 
@@ -571,7 +577,7 @@ class DebugClientBase(object):
                         res = exc.code
                         atexit._run_exitfuncs()
                     self.writestream.flush()
-                    self.progTerminated(res, exit=True)
+                    self.progTerminated(res)
                 return
 
             if cmd == DebugProtocol.RequestProfile:
@@ -581,7 +587,7 @@ class DebugClientBase(object):
                 wd, fn, args, erase = arg.split('|')
                 self.__setCoding(fn)
                 sys.argv.append(fn)
-                sys.argv.extend(eval(args.replace("u'", "'")))
+                sys.argv.extend(eval(args))
                 sys.path = self.__getSysPath(os.path.dirname(sys.argv[0]))
                 if wd == '':
                     os.chdir(sys.path[1])
@@ -591,6 +597,7 @@ class DebugClientBase(object):
                 # set the system exception handling function to ensure, that
                 # we report on all unhandled exceptions
                 sys.excepthook = self.__unhandled_exception
+                self.__interceptSignals()
                 
                 # generate a profile object
                 self.prof = PyProfile.PyProfile(sys.argv[0])
@@ -616,7 +623,7 @@ class DebugClientBase(object):
                         atexit._run_exitfuncs()
                     self.prof.save()
                     self.writestream.flush()
-                    self.progTerminated(res, exit=True)
+                    self.progTerminated(res)
                 return
 
             if cmd == DebugProtocol.RequestCoverage:
@@ -625,7 +632,7 @@ class DebugClientBase(object):
                 wd, fn, args, erase = arg.split('@@')
                 self.__setCoding(fn)
                 sys.argv.append(fn)
-                sys.argv.extend(eval(args.replace("u'", "'")))
+                sys.argv.extend(eval(args))
                 sys.path = self.__getSysPath(os.path.dirname(sys.argv[0]))
                 if wd == '':
                     os.chdir(sys.path[1])
@@ -635,13 +642,13 @@ class DebugClientBase(object):
                 # set the system exception handling function to ensure, that
                 # we report on all unhandled exceptions
                 sys.excepthook = self.__unhandled_exception
+                self.__interceptSignals()
                 
                 # generate a coverage object
                 self.cover = coverage(
                     auto_data=True,
                     data_file="{0}.coverage".format(
                         os.path.splitext(sys.argv[0])[0]))
-                self.cover.use_cache(True)
                 
                 if int(erase):
                     self.cover.erase()
@@ -667,7 +674,7 @@ class DebugClientBase(object):
                     self.cover.stop()
                     self.cover.save()
                     self.writestream.flush()
-                    self.progTerminated(res, exit=True)
+                    self.progTerminated(res)
                 return
 
             if cmd == DebugProtocol.RequestShutdown:
@@ -770,7 +777,8 @@ class DebugClientBase(object):
                     value = eval(
                         arg,
                         self.currentThread.getCurrentFrame().f_globals,
-                        self.currentThread.getCurrentFrameLocals())
+                        self.currentThread.getFrameLocals(self.framenr))
+                    self.currentThread.storeFrameLocals(self.framenr)
                 except:
                     # Report the exception and the traceback
                     try:
@@ -801,10 +809,11 @@ class DebugClientBase(object):
             
             if cmd == DebugProtocol.RequestExec:
                 _globals = self.currentThread.getCurrentFrame().f_globals
-                _locals = self.currentThread.getCurrentFrameLocals()
+                _locals = self.currentThread.getFrameLocals(self.framenr)
                 try:
                     code = compile(arg + '\n', '<stdin>', 'single')
                     exec(code, _globals, _locals)
+                    self.currentThread.storeFrameLocals(self.framenr)
                 except:
                     # Report the exception and the traceback
                     try:
@@ -859,6 +868,7 @@ class DebugClientBase(object):
                 # set the system exception handling function to ensure, that
                 # we report on all unhandled exceptions
                 sys.excepthook = self.__unhandled_exception
+                self.__interceptSignals()
                 
                 try:
                     import unittest
@@ -888,7 +898,6 @@ class DebugClientBase(object):
                         auto_data=True,
                         data_file="{0}.coverage".format(
                             os.path.splitext(covname)[0]))
-                    self.cover.use_cache(True)
                     if int(erase):
                         self.cover.erase()
                 else:
@@ -976,7 +985,8 @@ class DebugClientBase(object):
                                     frmnr -= 1
                                 _globals = cf.f_globals
                                 _locals = \
-                                    self.currentThread.getCurrentFrameLocals()
+                                    self.currentThread.getFrameLocals(
+                                        self.framenr)
                         # reset sys.stdout to our redirector (unconditionally)
                         if "sys" in _globals:
                             __stdout = _globals["sys"].stdout
@@ -990,6 +1000,8 @@ class DebugClientBase(object):
                             _locals["sys"].stdout = __stdout
                         else:
                             exec(code, _globals, _locals)
+                        
+                        self.currentThread.storeFrameLocals(self.framenr)
                 except SystemExit as exc:
                     self.progTerminated(exc.code)
                 except:
@@ -1040,7 +1052,7 @@ class DebugClientBase(object):
 
     def __interact(self):
         """
-        Private method to Interact with  the debugger.
+        Private method to interact with the debugger.
         """
         global DebugClientInstance
 
@@ -1064,6 +1076,9 @@ class DebugClientBase(object):
         while self.eventExit is None:
             wrdy = []
 
+            if self.writestream.nWriteErrors > self.writestream.maxtries:
+                break
+            
             if AsyncPendingWrite(self.writestream):
                 wrdy.append(self.writestream)
 
@@ -1140,23 +1155,11 @@ class DebugClientBase(object):
         @param redirect flag indicating redirection of stdin, stdout and
             stderr (boolean)
         """
-        if remoteAddress is None:                       # default: 127.0.0.1
-            sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
-            sock.connect((DebugProtocol.DebugAddress, port))
-        else:
-            if "@@i" in remoteAddress:
-                remoteAddress, index = remoteAddress.split("@@i")
-            else:
-                index = 0
-            if ":" in remoteAddress:                    # IPv6
-                sockaddr = socket.getaddrinfo(
-                    remoteAddress, port, 0, 0, socket.SOL_TCP)[0][-1]
-                sock = socket.socket(socket.AF_INET6, socket.SOCK_STREAM)
-                sockaddr = sockaddr[:-1] + (int(index),)
-                sock.connect(sockaddr)
-            else:                                       # IPv4
-                sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
-                sock.connect((remoteAddress, port))
+        if remoteAddress is None:
+            remoteAddress = "127.0.0.1"
+        elif "@@i" in remoteAddress:
+            remoteAddress = remoteAddress.split("@@i")[0]
+        sock = socket.create_connection((remoteAddress, port))
 
         self.readstream = AsyncFile(sock, sys.stdin.mode, sys.stdin.name)
         self.writestream = AsyncFile(sock, sys.stdout.mode, sys.stdout.name)
@@ -1181,6 +1184,61 @@ class DebugClientBase(object):
         """
         self.mainThread.user_exception(None, (exctype, excval, exctb), True)
     
+    def __interceptSignals(self):
+        """
+        Private method to intercept common signals.
+        """
+        for signum in [
+            signal.SIGABRT,                 # abnormal termination
+            signal.SIGFPE,                  # floating point exception
+            signal.SIGILL,                  # illegal instruction
+            signal.SIGSEGV,                 # segmentation violation
+        ]:
+            signal.signal(signum, self.__signalHandler)
+    
+    def __signalHandler(self, signalNumber, stackFrame):
+        """
+        Private method to handle signals.
+        
+        @param signalNumber number of the signal to be handled
+        @type int
+        @param stackFrame current stack frame
+        @type frame object
+        """
+        if signalNumber == signal.SIGABRT:
+            message = "Abnormal Termination"
+        elif signalNumber == signal.SIGFPE:
+            message = "Floating Point Exception"
+        elif signalNumber == signal.SIGILL:
+            message = "Illegal Instruction"
+        elif signalNumber == signal.SIGSEGV:
+            message = "Segmentation Violation"
+        else:
+            message = "Unknown Signal '{0}'".format(signalNumber)
+        
+        filename = self.absPath(stackFrame)
+        
+        linenr = stackFrame.f_lineno
+        ffunc = stackFrame.f_code.co_name
+        
+        if ffunc == '?':
+            ffunc = ''
+        
+        if ffunc and not ffunc.startswith("<"):
+            argInfo = getargvalues(stackFrame)
+            try:
+                fargs = formatargvalues(
+                    argInfo.args, argInfo.varargs,
+                    argInfo.keywords, argInfo.locals)
+            except Exception:
+                fargs = ""
+        else:
+            fargs = ""
+        
+        siglist = [message, [filename, linenr, ffunc, fargs]]
+        
+        self.write("{0}{1}".format(DebugProtocol.ResponseSignal, str(siglist)))
+    
     def absPath(self, fn):
         """
         Public method to convert a filename to an absolute name.
@@ -1254,13 +1312,12 @@ class DebugClientBase(object):
         """
         return self.running
 
-    def progTerminated(self, status, exit=False):
+    def progTerminated(self, status):
         """
         Public method to tell the debugger that the program has terminated.
         
         @param status return status
-        @param exit flag indicating to perform a sys.exit()
-        @type bool
+        @type int
         """
         if status is None:
             status = 0
@@ -1274,8 +1331,6 @@ class DebugClientBase(object):
             self.set_quit()
             self.running = None
             self.write('{0}{1:d}\n'.format(DebugProtocol.ResponseExit, status))
-            if exit:
-                sys.exit(status)
         
         # reset coding
         self.__coding = self.defaultCoding
@@ -1309,12 +1364,11 @@ class DebugClientBase(object):
                 scope = -1
         elif scope:
             dict = f.f_globals
+        elif f.f_globals is f.f_locals:
+                scope = -1
         else:
             dict = f.f_locals
             
-            if f.f_globals is f.f_locals:
-                scope = -1
-        
         varlist = [scope]
         
         if scope != -1:
@@ -1354,11 +1408,10 @@ class DebugClientBase(object):
                 scope = -1
         elif scope:
             dict = f.f_globals
+        elif f.f_globals is f.f_locals:
+                scope = -1
         else:
             dict = f.f_locals
-            
-            if f.f_globals is f.f_locals:
-                scope = -1
         
         varlist = [scope, var]
         
@@ -1383,7 +1436,7 @@ class DebugClientBase(object):
                     udict = dict
                 ndict = {}
                 # this has to be in line with VariablesViewer.indicators
-                if var[i][-2:] in ["[]", "()", "{}"]:
+                if var[i][-2:] in ["[]", "()", "{}"]:       # __IGNORE_WARNING__
                     if i + 1 == len(var):
                         if var[i][:-2] == '...':
                             dictkeys = [var[i - 1]]
@@ -1412,7 +1465,7 @@ class DebugClientBase(object):
                                 else:
                                     access = '{0!s}[{1!s}]'.format(
                                         access, var[i][:-2])
-                        if var[i][-2:] == "{}":
+                        if var[i][-2:] == "{}":     # __IGNORE_WARNING__
                             isDict = True
                         break
                     else:
@@ -1539,7 +1592,7 @@ class DebugClientBase(object):
                     exec('qvar = udict{0!s}'.format(access), globals(), loc)
                     qvar = loc["qvar"]
                 # this has to be in line with VariablesViewer.indicators
-                elif rvar and rvar[0][-2:] in ["[]", "()", "{}"]:
+                elif rvar and rvar[0][-2:] in ["[]", "()", "{}"]:   # __IGNORE_WARNING__
                     loc = {"udict": udict}
                     exec('qvar = udict["{0!s}"][{1!s}]'.format(rvar[0][:-2],
                          rvar[1]),
@@ -1561,7 +1614,7 @@ class DebugClientBase(object):
                              globals(), loc)
                         qvar = loc["qvar"]
                     # this has to be in line with VariablesViewer.indicators
-                    elif rvar and rvar[0][-2:] in ["[]", "()", "{}"]:
+                    elif rvar and rvar[0][-2:] in ["[]", "()", "{}"]:   # __IGNORE_WARNING__
                         loc = {"udict": udict}
                         exec('qvar = udict["{0!s}"][{1!s}]'.format(
                              rvar[0][:-2], rvar[1]), globals(), loc)
@@ -1885,8 +1938,7 @@ class DebugClientBase(object):
         """
         completerDelims = ' \t\n`~!@#$%^&*()-=+[{]}\\|;:\'",<>/?'
         
-        completions = []
-        state = 0
+        completions = set()
         # find position of last delim character
         pos = -1
         while pos >= -len(text):
@@ -1898,20 +1950,38 @@ class DebugClientBase(object):
                 break
             pos -= 1
         
+        # Get local and global completions
         try:
-            comp = self.complete(text, state)
-        except:
-            comp = None
-        while comp is not None:
-            completions.append(comp)
-            state += 1
-            try:
-                comp = self.complete(text, state)
-            except:
-                comp = None
+            localdict = self.currentThread.getFrameLocals(self.framenr)
+            localCompleter = Completer(localdict).complete
+            self.__getCompletionList(text, localCompleter, completions)
+        except AttributeError:
+            pass
+        self.__getCompletionList(text, self.complete, completions)
         
         self.write("{0}{1}||{2}\n".format(DebugProtocol.ResponseCompletion,
-                                          str(completions), text))
+                                          str(list(completions)), text))
+
+    def __getCompletionList(self, text, completer, completions):
+        """
+        Private method to create a completions list.
+        
+        @param text text to complete (string)
+        @param completer completer methode
+        @param completions set where to add new completions strings (set)
+        """
+        state = 0
+        try:
+            comp = completer(text, state)
+        except Exception:
+            comp = None
+        while comp is not None:
+            completions.add(comp)
+            state += 1
+            try:
+                comp = completer(text, state)
+            except Exception:
+                comp = None
 
     def startDebugger(self, filename=None, host=None, port=None,
                       enableTrace=True, exceptions=True, tracePython=False,
@@ -1965,6 +2035,7 @@ class DebugClientBase(object):
         # set the system exception handling function to ensure, that
         # we report on all unhandled exceptions
         sys.excepthook = self.__unhandled_exception
+        self.__interceptSignals()
         
         # now start debugging
         if enableTrace:
@@ -2022,6 +2093,7 @@ class DebugClientBase(object):
         # set the system exception handling function to ensure, that
         # we report on all unhandled exceptions
         sys.excepthook = self.__unhandled_exception
+        self.__interceptSignals()
         
         # This will eventually enter a local event loop.
         # Note the use of backquotes to cause a repr of self.running. The
@@ -2217,3 +2289,6 @@ class DebugClientBase(object):
         sysPath.insert(0, firstEntry)
         sysPath.insert(0, '')
         return sysPath
+
+#
+# eflag: noqa = M702
