@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright (c) 2007 - 2015 Detlev Offenbach <detlev@die-offenbachs.de>
+# Copyright (c) 2007 - 2016 Detlev Offenbach <detlev@die-offenbachs.de>
 #
 
 """
@@ -21,6 +21,7 @@ from PyQt5.QtNetwork import QNetworkAccessManager, QNetworkRequest, \
     QNetworkReply
 
 from E5Gui import E5MessageBox
+from E5Gui.E5Application import e5App
 
 from E5Network.E5NetworkProxyFactory import proxyAuthenticationRequired
 try:
@@ -371,6 +372,13 @@ class PluginManager(QObject):
             self.__modulesCount += 1
             if reload_:
                 imp.reload(module)
+                self.initOnDemandPlugin(name)
+                try:
+                    pluginObject = self.__onDemandInactivePlugins[name]
+                    pluginObject.initToolbar(
+                        self.__ui, e5App().getObject("ToolbarManager"))
+                except (KeyError, AttributeError):
+                    pass
         except PluginLoadError:
             print("Error loading plug-in module:", name)
         except PluginPy2IncompatibleError:
@@ -400,12 +408,22 @@ class PluginManager(QObject):
         
         if name in self.__inactiveModules:
             try:
+                pluginObject = self.__inactivePlugins[name]
+                try:
+                    pluginObject.prepareUnload()
+                except AttributeError:
+                    pass
                 del self.__inactivePlugins[name]
             except KeyError:
                 pass
             del self.__inactiveModules[name]
         elif name in self.__onDemandInactiveModules:
             try:
+                pluginObject = self.__onDemandInactivePlugins[name]
+                try:
+                    pluginObject.prepareUnload()
+                except AttributeError:
+                    pass
                 del self.__onDemandInactivePlugins[name]
             except KeyError:
                 pass
@@ -480,6 +498,21 @@ class PluginManager(QObject):
         except PluginActivationError:
             return
     
+    def initPluginToolbars(self, toolbarManager):
+        """
+        Public method to initialize plug-in toolbars.
+        
+        @param toolbarManager reference to the toolbar manager object
+            (E5ToolBarManager)
+        """
+        self.initOnDemandPlugins()
+        for pluginObject in self.__onDemandInactivePlugins.values():
+            try:
+                pluginObject.initToolbar(self.__ui, toolbarManager)
+            except AttributeError:
+                # ignore it
+                pass
+    
     def activatePlugins(self):
         """
         Public method to activate all plugins having the "autoactivate"
@@ -768,6 +801,9 @@ class PluginManager(QObject):
             module = self.__failedModules[name]
             autoactivate = False
             active = False
+        elif "_" in name:
+            # try stripping of a postfix
+            return self.getPluginDetails(name.rsplit("_", 1)[0])
         else:
             # should not happen
             return None
@@ -1115,6 +1151,7 @@ class PluginManager(QObject):
                 ).format(Preferences.getUI("PluginRepositoryUrl6"),
                          reply.errorString())
             )
+            reply.deleteLater()
             return
         
         ioDevice = QFile(self.pluginRepositoryFile + ".tmp")
@@ -1124,6 +1161,7 @@ class PluginManager(QObject):
         if QFile.exists(self.pluginRepositoryFile):
             QFile.remove(self.pluginRepositoryFile)
         ioDevice.rename(self.pluginRepositoryFile)
+        reply.deleteLater()
         
         if os.path.exists(self.pluginRepositoryFile):
             f = QFile(self.pluginRepositoryFile)
